#!/usr/bin/env python3

import json
import requests
import os
import argparse
from pprint import pprint as pp
from slugify import slugify

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-i",
        "--input",
        help="A data file following the MMD Metadata.",
        default="metadata.mmd",
        required=False,
    )
    parser.add_argument(
        "-c",
        "--ckan",
        help="The url of the CKAN instance.",
        default="https://reptor.mpcdf.mpg.de/api/action/",
        required=False,
    )

    parser.add_argument(
        "-t",
        "--title-field",
        help="Field name corresponding to the title of the dataset, NOT the title of the dataset.",
        default="Name",
        required=False,
    )

    parser.add_argument(
        "-o",
        "--organization",
        help="The CKAN organization to which the dataset is attributed",
        default="mpcdf",
        required=False,
    )
    print("")
    print("ARGUMENTS:")
    args = parser.parse_args()
    for k, v in args.__dict__.items():
        print(k + ": " + v)
    print("")

    with open(args.input, "r") as f:
        mmd_data = json.load(f)

    ckan_data = {}
    print("Data from {}".format(args.input) + ":")
    for key, value in mmd_data.items():
        if isinstance(value, str):
            print(key + ": " + str(value))
        elif isinstance(value, list):
            print(key + ":")
            for e in value:
                print(json.dumps(e, indent=2))
        if key != "id":
            ckan_data[key] = value

    print("_" * 10)
    print("")
    print("Now creating CKAN dataset.")
    print("_" * 10)
    print("")
    ckan_data["title"] = mmd_data[args.title_field]
    if "Name" in mmd_data:
        mmd_data.pop("Name")
    ckan_data["name"] = slugify(ckan_data["title"])
    ckan_data["owner_org"] = args.organization

    url = args.ckan + "/api/3/action/package_create"
    api_key = os.environ.get("CKAN_API_KEY")
    headers = {
        "Authorization": api_key,
        "Accept": "application/json",
        "Content-Type": "application/json",
    }

    print("DATA")
    print("-" * 10)
    for key, value in {**ckan_data}.items():
        if isinstance(value, str):
            print(key + ": " + str(value))
        elif isinstance(value, list):
            print(key, ":")
            for e in value:
                print(json.dumps(e, indent=2))
    print("")
    print("Request headers: ")
    print(json.dumps(headers, indent=2))

    r = requests.post(url, json={**ckan_data}, headers=headers)
    print("")
    print("Response content: ")
    try:
        print(json.dumps(r.json(), indent=2))
    except Error:
        print(r.text)
    print("")
    print("Status code: " + str(r.status_code))
    print("")
    print("Response headers: ")
    print(json.dumps(dict(r.headers), indent=2))
    print("")

if __name__ == "__main__":
    main()
